@@ -1,15 +1,6 @@
 // Copyright (c) 2023 Yuki Kimoto
 // MIT License
 
-#ifdef __linux__
-  // Enable X/Open System Interfaces (SUSv4) functions and POSIX.1-2008 standard functions
-  #define _XOPEN_SOURCE 700
-
-  // Enable BSD and System V extensions
-  #define _DEFAULT_SOURCE
-#endif
-
-
 #include "spvm_native.h"
 
 #include <stdlib.h>
@@ -33,14 +24,13 @@ int32_t SPVM__Sys__OS__is_windows(SPVM_ENV* env, SPVM_VALUE* stack) {
 int32_t SPVM__Sys__OS__defined(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   void* obj_macro_name = stack[0].oval;
-  
-  void* obj_value_ref = stack[1].oval;
-  
   if (!obj_macro_name) {
-    return env->die(env, stack, "The macro name $macro_name must be defined.", __func__, FILE_NAME, __LINE__);
+    return env->die(env, stack, "The $macro_name must be defined", __func__, FILE_NAME, __LINE__);
   }
   const char* macro_name = env->get_chars(env, stack, obj_macro_name);
   
+  void* obj_value_ref = stack[1].oval;
+  
   int32_t defined = 0;
   int32_t ival = 0;
   int64_t lval = 0;
@@ -247,27 +237,27 @@ int32_t SPVM__Sys__OS__defined(SPVM_ENV* env, SPVM_VALUE* stack) {
 #   endif
   }
   else {
-    return env->die(env, stack, "The macro name '%s' is not supported.", __func__, FILE_NAME, __LINE__, macro_name);
+    return env->die(env, stack, "The macro name \"%s\" is not supported yet", macro_name, __func__, FILE_NAME, __LINE__);
   }
   
   if (obj_value_ref) {
     
-    int32_t error_id = 0;
+    int32_t e = 0;
     
-    if (env->is_type(env, stack, obj_value_ref, env->get_basic_type_by_id(env, stack, SPVM_NATIVE_C_BASIC_TYPE_ID_INT), 1)) {
+    if (env->is_type(env, stack, obj_value_ref, SPVM_NATIVE_C_BASIC_TYPE_ID_INT, 1)) {
       int32_t* value_ref = env->get_elems_int(env, stack, obj_value_ref);
       *value_ref = ival;
     }
-    else if (env->is_type(env, stack, obj_value_ref, env->get_basic_type_by_id(env, stack, SPVM_NATIVE_C_BASIC_TYPE_ID_LONG), 1)) {
+    else if (env->is_type(env, stack, obj_value_ref, SPVM_NATIVE_C_BASIC_TYPE_ID_LONG, 1)) {
       int64_t* value_ref = env->get_elems_long(env, stack, obj_value_ref);
       *value_ref = lval;
     }
-    else if (env->is_type(env, stack, obj_value_ref, env->get_basic_type_by_id(env, stack, SPVM_NATIVE_C_BASIC_TYPE_ID_DOUBLE), 1)) {
+    else if (env->is_type(env, stack, obj_value_ref, SPVM_NATIVE_C_BASIC_TYPE_ID_DOUBLE, 1)) {
       double* value_ref = env->get_elems_double(env, stack, obj_value_ref);
       *value_ref = dval;
     }
     else {
-      return env->die(env, stack, "The array $value_ref to store a value must be the int[], long[], or double[] type.", __func__, FILE_NAME, __LINE__, macro_name);
+      return env->die(env, stack, "The $value_ref must be the int[], long[], or double[] type", macro_name, __func__, FILE_NAME, __LINE__);
     }
   }
   
