@@ -1,19 +1,10 @@
 // Copyright (c) 2023 Yuki Kimoto
 // MIT License
 
-#ifdef __linux__
-  // Enable X/Open System Interfaces (SUSv4) functions and POSIX.1-2008 standard functions
-  #define _XOPEN_SOURCE 700
-
-  // Enable BSD and System V extensions
-  #define _DEFAULT_SOURCE
-#endif
-
 #include "spvm_native.h"
 
 #include <time.h>
 #include <sys/time.h>
-#include <utime.h>
 #include <errno.h>
 
 #ifndef _WIN32
@@ -23,6 +14,8 @@
 static const char* FILE_NAME = "Sys/Time.c";
 
 int32_t SPVM__Sys__Time__time(SPVM_ENV* env, SPVM_VALUE* stack) {
+  (void)env;
+  (void)stack;
   
   int64_t epoch = (int64_t)time(NULL);
   
@@ -33,17 +26,10 @@ int32_t SPVM__Sys__Time__time(SPVM_ENV* env, SPVM_VALUE* stack) {
 
 int32_t SPVM__Sys__Time__localtime(SPVM_ENV* env, SPVM_VALUE* stack) {
   
-  int32_t error_id = 0;
+  int32_t e;
   
-  int64_t* time_ref = stack[0].lref;
-  
-  if (!time_ref) {
-    return env->die(env, stack, "The reference of the time $time_ref must be defined.", __func__, FILE_NAME, __LINE__);
-  }
-  
-  time_t time = *time_ref;
-  
-  struct tm* st_tm = env->new_memory_block(env, stack, sizeof(struct tm));
+  time_t time = (time_t)stack[0].lval;
+  struct tm* st_tm = env->new_memory_stack(env, stack, sizeof(struct tm));
   
 #ifdef _WIN32
   localtime_s(st_tm, &time);
@@ -51,8 +37,8 @@ int32_t SPVM__Sys__Time__localtime(SPVM_ENV* env, SPVM_VALUE* stack) {
   localtime_r(&time, st_tm);
 #endif
   
-  void* obj_time_info = env->new_pointer_object_by_name(env, stack, "Sys::Time::Tm", st_tm, &error_id, __func__, FILE_NAME, __LINE__);
-  if (error_id) { return error_id; }
+  void* obj_time_info = env->new_pointer_object_by_name(env, stack, "Sys::Time::Tm", st_tm, &e, __func__, FILE_NAME, __LINE__);
+  if (e) { return e; }
   
   stack[0].oval = obj_time_info;
   
@@ -61,17 +47,10 @@ int32_t SPVM__Sys__Time__localtime(SPVM_ENV* env, SPVM_VALUE* stack) {
 
 int32_t SPVM__Sys__Time__gmtime(SPVM_ENV* env, SPVM_VALUE* stack) {
   
-  int32_t error_id = 0;
+  int32_t e;
   
-  int64_t* time_ref = stack[0].lref;
-  
-  if (!time_ref) {
-    return env->die(env, stack, "The reference of the time $time_ref must be defined.", __func__, FILE_NAME, __LINE__);
-  }
-  
-  time_t time = *time_ref;
-  
-  struct tm* st_tm = env->new_memory_block(env, stack, sizeof(struct tm));
+  time_t time = (time_t)stack[0].lval;
+  struct tm* st_tm = env->new_memory_stack(env, stack, sizeof(struct tm));
   
 #ifdef _WIN32
   gmtime_s(st_tm, &time);
@@ -79,8 +58,8 @@ int32_t SPVM__Sys__Time__gmtime(SPVM_ENV* env, SPVM_VALUE* stack) {
   gmtime_r(&time, st_tm);
 #endif
   
-  void* obj_time_info = env->new_pointer_object_by_name(env, stack, "Sys::Time::Tm", st_tm, &error_id, __func__, FILE_NAME, __LINE__);
-  if (error_id) { return error_id; }
+  void* obj_time_info = env->new_pointer_object_by_name(env, stack, "Sys::Time::Tm", st_tm, &e, __func__, FILE_NAME, __LINE__);
+  if (e) { return e; }
   
   stack[0].oval = obj_time_info;
   
@@ -88,16 +67,18 @@ int32_t SPVM__Sys__Time__gmtime(SPVM_ENV* env, SPVM_VALUE* stack) {
 }
 
 int32_t SPVM__Sys__Time__gettimeofday(SPVM_ENV* env, SPVM_VALUE* stack) {
+  (void)env;
+  (void)stack;
   
   void* obj_tv = stack[0].oval;
   
-  void* obj_tz = stack[1].oval;
-  
   struct timeval* st_tv = NULL;
   if (obj_tv) {
     st_tv = env->get_pointer(env, stack, obj_tv);
   }
   
+  void* obj_tz = stack[1].oval;
+  
   struct timezone* st_tz = NULL;
   if (obj_tz) {
     st_tz = env->get_pointer(env, stack, obj_tz);
@@ -106,7 +87,7 @@ int32_t SPVM__Sys__Time__gettimeofday(SPVM_ENV* env, SPVM_VALUE* stack) {
   int32_t status = gettimeofday(st_tv, st_tz);
   
   if (status == -1) {
-    env->die(env, stack, "[System Error]gettimeofday() failed(%d: %s).", __func__, FILE_NAME, __LINE__, errno, env->strerror_nolen(env, stack, errno));
+    env->die(env, stack, "[System Error]gettimeofday failed:%s.", env->strerror(env, stack, errno, 0), __func__, FILE_NAME, __LINE__);
     return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
   }
   
@@ -116,11 +97,13 @@ int32_t SPVM__Sys__Time__gettimeofday(SPVM_ENV* env, SPVM_VALUE* stack) {
 }
 
 int32_t SPVM__Sys__Time__clock(SPVM_ENV* env, SPVM_VALUE* stack) {
+  (void)env;
+  (void)stack;
   
   int64_t cpu_time = clock();
   
   if (cpu_time == -1) {
-    env->die(env, stack, "[System Error]clock() failed(%d: %s).", __func__, FILE_NAME, __LINE__, errno, env->strerror_nolen(env, stack, errno));
+    env->die(env, stack, "[System Error]clock failed:%s.", env->strerror(env, stack, errno, 0), __func__, FILE_NAME, __LINE__);
     return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
   }
   
@@ -130,6 +113,8 @@ int32_t SPVM__Sys__Time__clock(SPVM_ENV* env, SPVM_VALUE* stack) {
 }
 
 int32_t SPVM__Sys__Time__clock_gettime(SPVM_ENV* env, SPVM_VALUE* stack) {
+  (void)env;
+  (void)stack;
   
   int32_t clk_id = stack[0].ival;
   
@@ -140,13 +125,13 @@ int32_t SPVM__Sys__Time__clock_gettime(SPVM_ENV* env, SPVM_VALUE* stack) {
     st_tp = env->get_pointer(env, stack, obj_tp);
   }
   else {
-    return env->die(env, stack, "$tp must be defined.", __func__, FILE_NAME, __LINE__);
+    return env->die(env, stack, "The $tp must be defined", __func__, FILE_NAME, __LINE__);
   }
   
   int32_t status = clock_gettime(clk_id, st_tp);
   
   if (status == -1) {
-    env->die(env, stack, "[System Error]clock_gettime() failed(%d: %s).", __func__, FILE_NAME, __LINE__, errno, env->strerror_nolen(env, stack, errno));
+    env->die(env, stack, "[System Error]clock_gettime failed:%s.", env->strerror(env, stack, errno, 0), __func__, FILE_NAME, __LINE__);
     return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
   }
   
@@ -156,6 +141,8 @@ int32_t SPVM__Sys__Time__clock_gettime(SPVM_ENV* env, SPVM_VALUE* stack) {
 }
 
 int32_t SPVM__Sys__Time__clock_getres(SPVM_ENV* env, SPVM_VALUE* stack) {
+  (void)env;
+  (void)stack;
   
   int32_t clk_id = stack[0].ival;
   
@@ -166,13 +153,13 @@ int32_t SPVM__Sys__Time__clock_getres(SPVM_ENV* env, SPVM_VALUE* stack) {
     st_res = env->get_pointer(env, stack, obj_res);
   }
   else {
-    return env->die(env, stack, "The resolution time $res must be defined.", __func__, FILE_NAME, __LINE__);
+    return env->die(env, stack, "The $res must be defined", __func__, FILE_NAME, __LINE__);
   }
   
   int32_t status = clock_getres(clk_id, st_res);
   
   if (status == -1) {
-    env->die(env, stack, "[System Error]clock_getres() failed(%d: %s).", __func__, FILE_NAME, __LINE__, errno, env->strerror_nolen(env, stack, errno));
+    env->die(env, stack, "[System Error]clock_getres failed:%s.", env->strerror(env, stack, errno, 0), __func__, FILE_NAME, __LINE__);
     return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
   }
   
@@ -183,24 +170,24 @@ int32_t SPVM__Sys__Time__clock_getres(SPVM_ENV* env, SPVM_VALUE* stack) {
 
 int32_t SPVM__Sys__Time__setitimer(SPVM_ENV* env, SPVM_VALUE* stack) {
 #if defined(_WIN32)
-  env->die(env, stack, "Sys::User#getitimer method is not supported in this system(defined(_WIN32)).", __func__, FILE_NAME, __LINE__);
+  env->die(env, stack, "getitimer is not supported on this system(defined(_WIN32))", __func__, FILE_NAME, __LINE__);
   return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_NOT_SUPPORTED_CLASS;
 #else
+  (void)env;
+  (void)stack;
   
   int32_t which = stack[0].ival;
   
   void* obj_new_value = stack[1].oval;
-  
-  void* obj_old_value = stack[2].oval;
-  
   struct itimerval* st_new_value = NULL;
   if (obj_new_value) {
     st_new_value = env->get_pointer(env, stack, obj_new_value);
   }
   else {
-    return env->die(env, stack, "The new timer $new_value must be defined.", __func__, FILE_NAME, __LINE__);
+    return env->die(env, stack, "The $new_value must be defined", __func__, FILE_NAME, __LINE__);
   }
-  
+
+  void* obj_old_value = stack[1].oval;
   struct itimerval* st_old_value = NULL;
   if (obj_old_value) {
     st_old_value = env->get_pointer(env, stack, obj_old_value);
@@ -209,7 +196,7 @@ int32_t SPVM__Sys__Time__setitimer(SPVM_ENV* env, SPVM_VALUE* stack) {
   int32_t status = setitimer(which, st_new_value, st_old_value);
   
   if (status == -1) {
-    env->die(env, stack, "[System Error]setitimer() failed(%d: %s).", __func__, FILE_NAME, __LINE__, errno, env->strerror_nolen(env, stack, errno));
+    env->die(env, stack, "[System Error]setitimer failed:%s.", env->strerror(env, stack, errno, 0), __func__, FILE_NAME, __LINE__);
     return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
   }
   
@@ -221,9 +208,11 @@ int32_t SPVM__Sys__Time__setitimer(SPVM_ENV* env, SPVM_VALUE* stack) {
 
 int32_t SPVM__Sys__Time__getitimer(SPVM_ENV* env, SPVM_VALUE* stack) {
 #if defined(_WIN32)
-  env->die(env, stack, "Sys::User#getitimer method is not supported in this system(defined(_WIN32)).", __func__, FILE_NAME, __LINE__);
+  env->die(env, stack, "getitimer is not supported on this system(defined(_WIN32))", __func__, FILE_NAME, __LINE__);
   return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_NOT_SUPPORTED_CLASS;
 #else
+  (void)env;
+  (void)stack;
   
   int32_t which = stack[0].ival;
   
@@ -234,13 +223,13 @@ int32_t SPVM__Sys__Time__getitimer(SPVM_ENV* env, SPVM_VALUE* stack) {
     st_curr_value = env->get_pointer(env, stack, obj_curr_value);
   }
   else {
-    return env->die(env, stack, "$curr_value must be defined.", __func__, FILE_NAME, __LINE__);
+    return env->die(env, stack, "The $curr_value must be defined", __func__, FILE_NAME, __LINE__);
   }
   
   int32_t status = getitimer(which, st_curr_value);
   
   if (status == -1) {
-    env->die(env, stack, "[System Error]getitimer() failed(%d: %s).", __func__, FILE_NAME, __LINE__, errno, env->strerror_nolen(env, stack, errno));
+    env->die(env, stack, "[System Error]getitimer failed:%s.", env->strerror(env, stack, errno, 0), __func__, FILE_NAME, __LINE__);
     return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
   }
   
@@ -252,13 +241,13 @@ int32_t SPVM__Sys__Time__getitimer(SPVM_ENV* env, SPVM_VALUE* stack) {
 
 int32_t SPVM__Sys__Time__times(SPVM_ENV* env, SPVM_VALUE* stack) {
 #if defined(_WIN32)
-  env->die(env, stack, "Sys::User#times method is not supported in this system(defined(_WIN32)).", __func__, FILE_NAME, __LINE__);
+  env->die(env, stack, "times is not supported on this system(defined(_WIN32))", __func__, FILE_NAME, __LINE__);
   return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_NOT_SUPPORTED_CLASS;
 #else
   void* obj_tms = stack[0].oval;
   
   if (!obj_tms) {
-    return env->die(env, stack, "$tms must be defined.", __func__, FILE_NAME, __LINE__);
+    return env->die(env, stack, "The $tms must be defined", __func__, FILE_NAME, __LINE__);
   }
   
   struct tms* st_tms = env->get_pointer(env, stack, obj_tms);
@@ -267,7 +256,7 @@ int32_t SPVM__Sys__Time__times(SPVM_ENV* env, SPVM_VALUE* stack) {
   int64_t clock_tick = times(st_tms);
   
   if (errno != 0) {
-    env->die(env, stack, "[System Error]times() failed(%d: %s).", __func__, FILE_NAME, __LINE__, errno, env->strerror_nolen(env, stack, errno));
+    env->die(env, stack, "[System Error]times failed:%s.", env->strerror(env, stack, errno, 0), __func__, FILE_NAME, __LINE__);
     return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
   }
   
@@ -279,18 +268,20 @@ int32_t SPVM__Sys__Time__times(SPVM_ENV* env, SPVM_VALUE* stack) {
 
 int32_t SPVM__Sys__Time__clock_nanosleep(SPVM_ENV* env, SPVM_VALUE* stack) {
 #ifdef __APPLE__
-  env->die(env, stack, "Sys::User#clock_nanosleep method is not supported in this system(__APPLE__).", __func__, FILE_NAME, __LINE__);
+  env->die(env, stack, "clock_nanosleep is not supported on this system(__APPLE__)", __func__, FILE_NAME, __LINE__);
   return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_NOT_SUPPORTED_CLASS;
 #elif __FreeBSD__ && !(__FreeBSD__ >= 13)
-  env->die(env, stack, "Sys::User#clock_nanosleep method is not supported in this system(__FreeBSD__ && !(__FreeBSD__ >= 13)).", __func__, FILE_NAME, __LINE__);
+  env->die(env, stack, "clock_nanosleep is not supported on this system(__FreeBSD__ && !(__FreeBSD__ >= 13))", __func__, FILE_NAME, __LINE__);
   return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_NOT_SUPPORTED_CLASS;
 #elif __OpenBSD__
-  env->die(env, stack, "Sys::User#clock_nanosleep method is not supported in this system(__OpenBSD__).", __func__, FILE_NAME, __LINE__);
+  env->die(env, stack, "clock_nanosleep is not supported on this system(__OpenBSD__)", __func__, FILE_NAME, __LINE__);
   return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_NOT_SUPPORTED_CLASS;
 #else
+  (void)env;
+  (void)stack;
   
   int32_t clockid = stack[0].ival;
-  
+
   int32_t flags = stack[1].ival;
   
   void* obj_request = stack[2].oval;
@@ -300,7 +291,7 @@ int32_t SPVM__Sys__Time__clock_nanosleep(SPVM_ENV* env, SPVM_VALUE* stack) {
     st_request = env->get_pointer(env, stack, obj_request);
   }
   else {
-    return env->die(env, stack, "The request time $request must be defined.", __func__, FILE_NAME, __LINE__);
+    return env->die(env, stack, "The $request must be defined", __func__, FILE_NAME, __LINE__);
   }
   
   void* obj_remain = stack[3].oval;
@@ -311,9 +302,9 @@ int32_t SPVM__Sys__Time__clock_nanosleep(SPVM_ENV* env, SPVM_VALUE* stack) {
   }
   
   int32_t ret_errno = clock_nanosleep(clockid, flags, st_request, st_remain);
-  
+
   if (ret_errno != 0) {
-    env->die(env, stack, "[System Error]clock_nanosleep() failed(%d: %s).", __func__, FILE_NAME, __LINE__, ret_errno, env->strerror_nolen(env, stack, ret_errno));
+    env->die(env, stack, "[System Error]clock_nanosleep failed:%s.", env->strerror(env, stack, ret_errno, 0), __func__, FILE_NAME, __LINE__);
     return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
   }
   
@@ -324,119 +315,35 @@ int32_t SPVM__Sys__Time__clock_nanosleep(SPVM_ENV* env, SPVM_VALUE* stack) {
 }
 
 int32_t SPVM__Sys__Time__nanosleep(SPVM_ENV* env, SPVM_VALUE* stack) {
+  (void)env;
+  (void)stack;
   
   void* obj_rqtp = stack[0].oval;
   
-  void* obj_rmtp = stack[1].oval;
-  
   struct timespec* st_rqtp = NULL;
   if (obj_rqtp) {
     st_rqtp = env->get_pointer(env, stack, obj_rqtp);
   }
   else {
-    return env->die(env, stack, "The request time $rqtp must be defined.", __func__, FILE_NAME, __LINE__);
+    return env->die(env, stack, "The $rqtp must be defined", __func__, FILE_NAME, __LINE__);
   }
   
+  void* obj_rmtp = stack[1].oval;
+  
   struct timespec* st_rmtp = NULL;
   if (obj_rmtp) {
     st_rmtp = env->get_pointer(env, stack, obj_rmtp);
   }
   
   int32_t status = nanosleep(st_rqtp, st_rmtp);
-  
+
   if (status == -1) {
-    env->die(env, stack, "[System Error]nanosleep() failed(%d: %s).", __func__, FILE_NAME, __LINE__, errno, env->strerror_nolen(env, stack, errno));
+    env->die(env, stack, "[System Error]nanosleep failed:%s.", env->strerror(env, stack, errno, 0), __func__, FILE_NAME, __LINE__);
     return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
   }
-  
+
   stack[0].ival = status;
   
   return 0;
 }
 
-int32_t SPVM__Sys__Time__utime(SPVM_ENV* env, SPVM_VALUE* stack) {
-  
-  int32_t error_id = 0;
-  
-  void* obj_filename = stack[0].oval;
-  
-  void* obj_times = stack[1].oval;
-  
-  if (!obj_filename) {
-    return env->die(env, stack, "The file path $filename must be defined.", __func__, FILE_NAME, __LINE__);
-  }
-  const char* filename = env->get_chars(env, stack, obj_filename);
-  
-  struct utimbuf* st_times;
-  if (obj_times) {
-    st_times = env->get_pointer(env, stack, obj_times);
-  }
-  else {
-    st_times = NULL;
-  }
-  
-  int32_t status = utime(filename, st_times);
-  if (status == -1) {
-    env->die(env, stack, "[System Error]utime() failed(%d: %s). $filename='%s'.", __func__, FILE_NAME, __LINE__, errno, env->strerror_nolen(env, stack, errno), filename);
-    return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
-  }
-  
-  stack[0].ival = status;
-  
-  return 0;
-}
-
-int32_t SPVM__Sys__Time__utimes(SPVM_ENV* env, SPVM_VALUE* stack) {
-#if defined(_WIN32)
-  env->die(env, stack, "Sys::User#utimes method is not supported in this system(defined(_WIN32)).", __func__, FILE_NAME, __LINE__);
-  return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_NOT_SUPPORTED_CLASS;
-#else
-  int32_t error_id = 0;
-  
-  void* obj_filename = stack[0].oval;
-  
-  void* obj_times = stack[1].oval;
-  
-  if (!obj_filename) {
-    return env->die(env, stack, "The file path $filename must be defined.", __func__, FILE_NAME, __LINE__);
-  }
-  const char* filename = env->get_chars(env, stack, obj_filename);
-  
-  if (!obj_times) {
-    return env->die(env, stack, "The utime infomation $times must be defined.", __func__, FILE_NAME, __LINE__);
-  }
-  
-  int32_t times_length = env->length(env, stack, obj_times);
-  
-  if (!(times_length == 2)) {
-    return env->die(env, stack, "The length of the utime infomation $times must be 2.", __func__, FILE_NAME, __LINE__);
-  }
-  
-  void* obj_times0 = env->get_elem_object(env, stack, obj_times, 0);
-  struct timeval* times0 = env->get_pointer(env, stack, obj_times0);
-  
-  void* obj_times1 = env->get_elem_object(env, stack, obj_times, 1);
-  struct timeval* times1 = env->get_pointer(env, stack, obj_times1);
-  
-  const struct timeval times[2] = {*times0, *times1};
-  
-  int32_t status = utimes(filename, times);
-  
-  if (status == -1) {
-    env->die(env, stack, "[System Error]utimes() failed(%d: %s).", __func__, FILE_NAME, __LINE__, errno, env->strerror_nolen(env, stack, errno), filename);
-    return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
-  }
-  
-  stack[0].ival = status;
-  
-  return 0;
-#endif
-}
-
-int32_t SPVM__Sys__Time__tzset(SPVM_ENV* env, SPVM_VALUE* stack) {
-  
-  tzset();
-  
-  return 0;
-}
-
